# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['klaam',
 'klaam.external',
 'klaam.external.FastSpeech2',
 'klaam.external.FastSpeech2.audio',
 'klaam.external.FastSpeech2.hifigan',
 'klaam.external.FastSpeech2.model',
 'klaam.external.FastSpeech2.phonetise',
 'klaam.external.FastSpeech2.preprocessor',
 'klaam.external.FastSpeech2.text',
 'klaam.external.FastSpeech2.transformer',
 'klaam.external.FastSpeech2.utils',
 'klaam.models',
 'klaam.processors',
 'klaam.speech_corpus',
 'klaam.utils']

package_data = \
{'': ['*'], 'klaam.external.FastSpeech2': ['img/*']}

install_requires = \
['Unidecode>=1.3.4,<2.0.0',
 'arabic-pronounce>=0.2.5,<0.3.0',
 'codernitydb3>=0.6.0,<0.7.0',
 'datasets>=2.1.0,<3.0.0',
 'ffmpeg>=1.4,<2.0',
 'gdown>=4.4.0,<5.0.0',
 'inflect>=5.5.2,<6.0.0',
 'jiwer>=2.3.0,<3.0.0',
 'jupyter>=1.0.0,<2.0.0',
 'librosa>=0.9.1,<0.10.0',
 'matplotlib>=3.5.1,<4.0.0',
 'mishkal>=0.4.1,<0.5.0',
 'numba>=0.55.1,<0.56.0',
 'pandas==1.3',
 'pypinyin>=0.46.0,<0.47.0',
 'torch>=1.11.0,<2.0.0',
 'torchaudio>=0.11.0,<0.12.0',
 'tqdm>=4.64.0,<5.0.0',
 'transformers>=4.18.0,<5.0.0']

setup_kwargs = {
    'name': 'klaam',
    'version': '0.0.1',
    'description': 'Arabic speech recognition, classification and text-to-speech.',
    'long_description': '# klaam\nArabic speech recognition, classification and text-to-speech using many advanced models like wave2vec and fastspeech2. This repository allows training and prediction using pretrained models.\n\n<p align="center">\n<img src="https://raw.githubusercontent.com/ARBML/klaam/main/misc/klaam_logo.png" width="250px"/>\n</p>\n\n\n## Usage\n\n### Speech Classification\n```python\nfrom klaam import SpeechClassification\nmodel = SpeechClassification()\nmodel.classify(wav_file)\n```\n\n### Speech Recongnition\n```python\nfrom klaam import SpeechRecognition\nmodel = SpeechRecognition()\nmodel.transcribe(wav_file)\n```\n\n\n### Text To Speech\n```python\nfrom klaam import TextToSpeech\nprepare_tts_model_path = "../cfgs/FastSpeech2/config/Arabic/preprocess.yaml"\nmodel_config_path = "../cfgs/FastSpeech2/config/Arabic/model.yaml"\ntrain_config_path = "../cfgs/FastSpeech2/config/Arabic/train.yaml"\nvocoder_config_path = "../cfgs/FastSpeech2/model_config/hifigan/config.json"\nspeaker_pre_trained_path = "../data/model_weights/hifigan/generator_universal.pth.tar"\n\nmodel = TextToSpeech(prepare_tts_model_path, model_config_path, train_config_path, vocoder_config_path, speaker_pre_trained_path)\n\nmodel.synthesize(sample_text)\n```\n\nThere are two avilable models for recognition trageting MSA and egyptian dialect . You can set any of them using the `lang` attribute\n\n```python\nfrom klaam import SpeechRecognition\nmodel = SpeechRecognition(lang = \'msa\')\nmodel.transcribe(\'file.wav\')\n```\n\n## Datasets\n\n| Dataset | Description | link |\n|---------| ------------------------------ | ---- |\n| MGB-3  | Egyptian Arabic Speech recognition in the wild. Every sentence was annotated by four annotators. More than 15 hours have been collected from YouTube.  |  requires registeration [here](https://arabicspeech.org/mgb3-asr/)|\n| ADI-5  | More than 50 hours collected from Aljazeera TV.  4 regional dialectal: Egyptian (EGY), Levantine (LAV), Gulf (GLF), North African (NOR), and Modern Standard Arabic (MSA). This dataset is a part of the MGB-3 challenge.  | requires registeration [here](https://arabicspeech.org/mgb3-adi/)|\n|Common voice | Multlilingual dataset avilable on huggingface | [here](https://github.com/huggingface/datasets/tree/master/datasets/common_voice). |\n|Arabic Speech Corpus | Arabic dataset with alignment and transcriptions | [here](http://en.arabicspeechcorpus.com/). |\n\n## Models\n\nWe currently support four models, three of them are avilable on transformers.\n\n|Language | Description | Source |\n|-------- | ----------- | ------ |\n|Egyptian | Speech recognition | [wav2vec2-large-xlsr-53-arabic-egyptian](https://huggingface.co/Zaid/wav2vec2-large-xlsr-53-arabic-egyptian)|\n|Standard Arabic | Speech recognition | [wav2vec2-large-xlsr-53-arabic](https://huggingface.co/elgeish/wav2vec2-large-xlsr-53-arabic)\n|EGY, NOR, LAV, GLF, MSA | Speech classification | [wav2vec2-large-xlsr-dialect-classification](https://huggingface.co/Zaid/wav2vec2-large-xlsr-dialect-classification)|\n|Standard Arabic| Text-to-Speech | [fastspeech2]()|\n\n## Example Notebooks\n<table>\n  <tr>\n    <th><b>Name</b></th>\n    <th><b>Description</b></th>\n    <th><b>Notebook</b></th>\n  </tr>\n\n  <tr>\n    <td>Demo</td>\n    <td>Classification, Recongition and Text-to-speech  in a few lines of code.</td>\n    <td><a href="https://colab.research.google.com/github/ARBML/klaam/blob/main/demo.ipynb">\n    <img src="https://colab.research.google.com/assets/colab-badge.svg"  >\n    </a></td>\n  </tr>\n\n  <tr>\n    <td>Demo with mic</td>\n    <td>Audio Recongition and classification with recording.</td>\n    <td><a href="https://colab.research.google.com/github/ARBML/klaam/blob/main/demo_with_mic.ipynb">\n    <img src="https://colab.research.google.com/assets/colab-badge.svg">\n    </a></td>\n  </tr>\n<table>\n\n## Training\n\nThe scripts are a modification of [jqueguiner/wav2vec2-sprint](https://github.com/jqueguiner/wav2vec2-sprint).\n\n### classification\nThis script is used for the classification task on the 5 classes.\n\n```sh\npython run_classifier.py \\\n    --model_name_or_path="facebook/wav2vec2-large-xlsr-53" \\\n    --output_dir=/path/to/output \\\n    --cache_dir=/path/to/cache/ \\\n    --freeze_feature_extractor \\\n    --num_train_epochs="50" \\\n    --per_device_train_batch_size="32" \\\n    --preprocessing_num_workers="1" \\\n    --learning_rate="3e-5" \\\n    --warmup_steps="20" \\\n    --evaluation_strategy="steps"\\\n    --save_steps="100" \\\n    --eval_steps="100" \\\n    --save_total_limit="1" \\\n    --logging_steps="100" \\\n    --do_eval \\\n    --do_train \\\n```\n\n### Recognition\n\nThis script is for training on the dataset for pretraining on the egyption dialects dataset.\n\n```sh\npython run_mgb3.py \\\n    --model_name_or_path="facebook/wav2vec2-large-xlsr-53" \\\n    --output_dir=/path/to/output \\\n    --cache_dir=/path/to/cache/ \\\n    --freeze_feature_extractor \\\n    --num_train_epochs="50" \\\n    --per_device_train_batch_size="32" \\\n    --preprocessing_num_workers="1" \\\n    --learning_rate="3e-5" \\\n    --warmup_steps="20" \\\n    --evaluation_strategy="steps"\\\n    --save_steps="100" \\\n    --eval_steps="100" \\\n    --save_total_limit="1" \\\n    --logging_steps="100" \\\n    --do_eval \\\n    --do_train \\\n```\n\nThis script can be used for Arabic common voice training\n\n```sh\npython run_common_voice.py \\\n    --model_name_or_path="facebook/wav2vec2-large-xlsr-53" \\\n    --dataset_config_name="ar" \\\n    --output_dir=/path/to/output/ \\\n    --cache_dir=/path/to/cache \\\n    --overwrite_output_dir \\\n    --num_train_epochs="1" \\\n    --per_device_train_batch_size="32" \\\n    --per_device_eval_batch_size="32" \\\n    --evaluation_strategy="steps" \\\n    --learning_rate="3e-4" \\\n    --warmup_steps="500" \\\n    --fp16 \\\n    --freeze_feature_extractor \\\n    --save_steps="10" \\\n    --eval_steps="10" \\\n    --save_total_limit="1" \\\n    --logging_steps="10" \\\n    --group_by_length \\\n    --feat_proj_dropout="0.0" \\\n    --layerdrop="0.1" \\\n    --gradient_checkpointing \\\n    --do_train --do_eval \\\n    --max_train_samples 100 --max_val_samples 100\n```\n\n### Text To Speech\n\nWe use the pytorch implementation of fastspeech2 by [ming024](https://github.com/ming024/FastSpeech2). The procedure is as follows\n\nDownload the dataset\n\n```\nwget http://en.arabicspeechcorpus.com/arabic-speech-corpus.zip\nunzip arabic-speech-corpus.zip\n```\n\nCreate multiple directories for data\n\n```\nmkdir -p raw_data/Arabic/Arabic preprocessed_data/Arabic/TextGrid/Arabic\ncp arabic-speech-corpus/textgrid/* preprocessed_data/Arabic/TextGrid/Arabic\n```\n\nPrepare metadata\n\n```python\nimport os\nbase_dir = \'/content/arabic-speech-corpus\'\nlines = []\nfor lab_file in os.listdir(f\'{base_dir}/lab\'):\n  lines.append(lab_file[:-4]+\'|\'+open(f\'{base_dir}/lab/{lab_file}\', \'r\').read())\n\n\nopen(f\'{base_dir}/metadata.csv\', \'w\').write((\'\\n\').join(lines))\n```\n\nClone my fork\n\n```bash\ngit clone --depth 1 https://github.com/zaidalyafeai/FastSpeech2\ncd FastSpeech2\npip install -r requirements.txt\n```\n\nPrepare alignments and prepreocessed data\n\n```\npython3 prepare_align.py config/Arabic/preprocess.yaml\npython3 preprocess.py config/Arabic/preprocess.yaml\n```\n\nUnzip vocoders\n\n```\nunzip hifigan/generator_LJSpeech.pth.tar.zip -d hifigan\nunzip hifigan/generator_universal.pth.tar.zip -d hifigan\n```\n\nStart training\n\n```\npython3 train.py -p config/Arabic/preprocess.yaml -m config/Arabic/model.yaml -t config/Arabic/train.yaml\n```\n',
    'author': 'Zaid Alyafeai',
    'author_email': 'alyafey22@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ARBML/klaam',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.7.13',
}


setup(**setup_kwargs)
